/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.fidentis.analyst.visitors.procrustes;

import cz.fidentis.analyst.face.HumanFace;
import cz.fidentis.analyst.mesh.MeshVisitor;
import cz.fidentis.analyst.mesh.core.MeshFacet;
import cz.fidentis.analyst.procrustes.ProcrustesTransformation;
import cz.fidentis.analyst.visitors.mesh.sampling.NoSampling;
import cz.fidentis.analyst.visitors.mesh.sampling.PointSampling;
import cz.fidentis.analyst.procrustes.ProcrustesAnalysis;

/**
 *
 * @author Jakub Kolman
 */
public class ProcrustesVisitor extends MeshVisitor {
    
    private final HumanFace primaryFace;
    private final HumanFace secondaryFace;
    private final PointSampling samplingStrategy;
    private final boolean scale;
    private ProcrustesTransformation transformation = new ProcrustesTransformation();

    /**
     * constructor 
     * 
     * @param primaryFace Face that is used as a from that a secondondary face will be matched to
     * @param secondaryFace Face for which are transformation calculated to be fitted to a primary face
     * @param scale Allows or forbids second face to be scaled
     * @param strategy 
     */
    public ProcrustesVisitor(HumanFace primaryFace, HumanFace secondaryFace, boolean scale, PointSampling strategy) {
        if (primaryFace == null || secondaryFace == null) {
            throw new IllegalArgumentException("Procrustes visitor: one of the faces is null");
        }
        this.primaryFace = primaryFace;
        this.secondaryFace = secondaryFace;
        this.scale = scale;
        this.samplingStrategy = (strategy == null) ? new NoSampling() : strategy;
    }
    
    /**
     * Constructor without PointSampling strategy set. 
     * For more information take a look on {@link #ProcrustesVisitor(cz.fidentis.analyst.face.HumanFace, cz.fidentis.analyst.face.HumanFace, boolean, cz.fidentis.analyst.visitors.mesh.sampling.PointSampling) }
     * 
     * 
     * @param primaryFace
     * @param secondaryFace
     * @param scale 
     */
    public ProcrustesVisitor(HumanFace primaryFace, HumanFace secondaryFace, boolean scale) {
        this(primaryFace, secondaryFace, scale, null);
    }
    
    public ProcrustesTransformation getTransformation() {
        return this.transformation;
    }
    
    @Override
    public void visitMeshFacet(MeshFacet transformedFacet) {
        try {
            ProcrustesAnalysis procrustes = new ProcrustesAnalysis(this.primaryFace, this.secondaryFace);
            this.transformation = procrustes.analyze();
        } catch (Exception e) {
            System.err.println(e.getMessage());
        }
    }
    
}
